#include "Playview.h"
#include <be/translation/TranslationUtils.h>
#include <stdio.h>
#include <stdlib.h>
#include <Window.h>
#include "Defines.h"

extern	BPath		GamePath;

PlayView::PlayView(BRect T) : BView(T,"Playview",B_FOLLOW_ALL_SIDES,B_FULL_UPDATE_ON_RESIZE|B_WILL_DRAW|B_PULSE_NEEDED)
{
	BPath	P(GamePath);
	P.Append("Gfx/SameGame.bmp");	

	printf("Attemting to load BMP file: %s\n",P.Path());

	m_Bitmap=BTranslationUtils::GetBitmap(P.Path());

	if(m_Bitmap==NULL)
	{
		printf("Error could not load Bitmap: %s\n",P.Path());
		printf("Bailing out!!!!!\n");
		exit(-1);
	}
	else
		printf("Loaded picture: %s\n",P.Path());
	
	SetViewColor(B_TRANSPARENT_COLOR);
	for(int a=0;a<15;a++)
		for(int b=0;b<10;b++)
		{	
			SelectedMatrix[a][b]=false;
			
			Matrix[a][b].m_Type=rand()%3;
			Matrix[a][b].m_Frame=0;
			Matrix[a][b].m_Dirty=true;
		}
		
		BPoint	T(0,0);
		m_MousePoint=T;
		m_MouseInside=false;
		m_Score=0;		
		m_Active=true;
}

void PlayView::WindowActivated(bool active)
{
	m_Active=active;

	//Dirty all entries in the matrix so that they will be redrawn
	if(active)
	{
		for(int a=0;a<15;a++)
			for(int b=0;b<10;b++)
				Matrix[a][b].m_Dirty=true;
	}
}

bool PlayView::CheckIfGameOver(void)
{
	for(int a=0;a<15;a++)
		for(int b=0;b<10;b++)
		{
			if(Matrix[a][b].m_Type!=-1)
			{
				if(a-1>=0)
					if(Matrix[a-1][b].m_Type==Matrix[a][b].m_Type)
						return(false);
				if(a+1<15)
					if(Matrix[a+1][b].m_Type==Matrix[a][b].m_Type)
						return(false);
				if(b-1>=0)
					if(Matrix[a][b-1].m_Type==Matrix[a][b].m_Type)
						return(false);
				if(b+1<10)
					if(Matrix[a][b+1].m_Type==Matrix[a][b].m_Type)
						return(false);
	
			}
		}
	return(true);
}

void PlayView::MouseDown(BPoint point) 
{
	if(m_SelectCount>1)		//This implements the userclick
	{
		int	ScoreCount=0;

		for(int a=0;a<15;a++)
			for(int b=0;b<10;b++)
			{
				if(SelectedMatrix[a][b])
				{
					Matrix[a][b].m_Type=-1;
					Matrix[a][b].m_Dirty=true;
					ScoreCount+=1;
				}
			}
		
		m_Score+=ScoreCount*ScoreCount*ScoreCount*5+200;	
		ShuffleMatrix();
		for(int a=0;a<15;a++)
			for(int b=0;b<10;b++)
			{
				SelectedMatrix[a][b]=false;
				Matrix[a][b].m_Dirty=true;
			}

		int	newx=(int)point.x/32;
		int	newy=(int)point.y/32;

			//Search all selected	
			m_SelectCount=0;
			if(Matrix[newx][newy].m_Type!=-1)
				FindAllSelected(newx,newy,Matrix[newx][newy].m_Type);
			//In case we found one entry, do not animate
			if(m_SelectCount==1)
				for(int a=0;a<15;a++)
					for(int b=0;b<10;b++)
						SelectedMatrix[a][b]=false;
		if(CheckIfGameOver())
		{
			BMessage*	M=new BMessage(MSG_GAMEOVER);
			M->AddInt32("Score",m_Score);
			BWindow*	P=Window();
			P->PostMessage(M);
		}
	}
	
}

void PlayView::Pulse()
{
	static	int delay=0;

	if(m_Active)
	{
	//First we get the mouse Coords

	delay++;
	if(delay==5)
		{ delay=0; if(m_Score>=10) m_Score-=10;}
	
	BPoint	Temp;
	uint32	Temp1;
	GetMouse(&Temp,&Temp1,false);
	int	newx=(int)Temp.x/32;
	int	newy=(int)Temp.y/32;
	
	if(Temp.x>=0&&Temp.x<=479&&Temp.y>=0&&Temp.y<=320)
	{
		m_MouseInside=true;
		int	oldx=(int)m_MousePoint.x/32;
		int	oldy=(int)m_MousePoint.y/32;
		if(oldx!=newx||oldy!=newy)
		{
			m_MousePoint=Temp;	//Move our idea of a cursor
			//Erase the selected matrix so that we rock
			for(int a=0;a<15;a++)
				for(int b=0;b<10;b++)
					SelectedMatrix[a][b]=false;
			//Search all selected	
			m_SelectCount=0;
			if(Matrix[newx][newy].m_Type!=-1)
				FindAllSelected(newx,newy,Matrix[newx][newy].m_Type);
			//In case we found one entry, do not animate
			if(m_SelectCount==1)
				for(int a=0;a<15;a++)
					for(int b=0;b<10;b++)
						SelectedMatrix[a][b]=false;
			
		}	
	}
	else
		m_MouseInside=false;	//No mouse
		

	
	if(m_MouseInside)
	{
		//Animate all selected buttons
		for(int a=0;a<15;a++)
			for(int b=0;b<10;b++)
			{
				if(SelectedMatrix[a][b])
				{
					Matrix[a][b].m_Frame=(Matrix[a][b].m_Frame+1)%7;
					Matrix[a][b].m_Dirty=true;
				}
			}
	}
	
	Invalidate();
	}
}

void PlayView::ShuffleMatrix(void)
{
	
	for(int x=0;x<15;x++)
	{
		for(int y=8;y>=0;y--)
		{
			if(Matrix[x][y+1].m_Type==-1&&Matrix[x][y].m_Type!=-1)
			{
				Matrix[x][y+1]=Matrix[x][y];
				Matrix[x][y].m_Type=-1;
				y=9;
			}
		}
	}


	for(int x=1;x<15;x++)
	{
		if(IsColumnEmpty(x-1)&&!IsColumnEmpty(x))
		{	
			CopyColumn(x,x-1);
			for(int y=0;y<10;y++)
				Matrix[x][y].m_Type=-1;
			x=0;
		}
	}

}

bool	PlayView::IsColumnEmpty(int Source)
{
		bool	IsEmpty=true;
		for(int y=0;y<10;y++)
			if(Matrix[Source][y].m_Type!=-1)
				IsEmpty=false;
		return(IsEmpty);
}
void	PlayView::CopyColumn(int Source,int Dest)
{
	for(int y=0;y<10;y++)
		Matrix[Dest][y]=Matrix[Source][y];
}

void PlayView::FindAllSelected(int x, int y,int type)
{

	//Check if it is the same color
	if(type==Matrix[x][y].m_Type)
	{
		SelectedMatrix[x][y]=true;	//Mark as visited
		m_SelectCount++;
			
		if(x-1>=0&&SelectedMatrix[x-1][y]==false)
			FindAllSelected(x-1,y,type);
		if(x+1<15&&SelectedMatrix[x+1][y]==false)
			FindAllSelected(x+1,y,type);
		
		if(y-1>=0&&SelectedMatrix[x][y-1]==false)
			FindAllSelected(x,y-1,type);

		if(y+1<10&&SelectedMatrix[x][y+1]==false)
			FindAllSelected(x,y+1,type);
	}	
}

void PlayView::Draw(BRect updateRect)
{
	SetHighColor(255,255,255);
	SetLowColor(0,0,0);

	BRect	T(0,320,480,359);
	FillRect(T);

	SetHighColor(0,0,0);
	SetLowColor(255,255,255);
	MovePenTo(10,330);
	
	char	T1[255];
	sprintf(T1,"Score: %d",m_Score);
	DrawString(T1);
	
	for(int a=0;a<15;a++)
		for(int b=0;b<10;b++)
		{	
				if(Matrix[a][b].m_Dirty)
				{
					if(Matrix[a][b].m_Type==-1)
					{
						BRect	Dest(a*32,b*32,a*32+31,b*32+31);
						FillRect(Dest);
						Matrix[a][b].m_Dirty=false;
					}
					else
					{
						float y=(float)Matrix[a][b].m_Type*32;
						float x=(float)Matrix[a][b].m_Frame*32;
						BRect	Source(x,y,x+31,y+31);
					
						BRect	Dest(a*32,b*32,a*32+31,b*32+31);
						DrawBitmapAsync(m_Bitmap,Source,Dest);
						Matrix[a][b].m_Dirty=false;
					}
				}
		}
	Sync();

}